unit BrowseMountPoint;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, volume, ImgList, Dialogs;

const FILE_ATTRIBUTE_REPARSE_POINT    = $00000400;
const FILE_SUPPORTS_REPARSE_POINTS    = $00000080;
const IO_REPARSE_TAG_MOUNT_POINT      = $A0000003;


type
  TBrowsePathDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Bevel1: TBevel;
    DirTree: TTreeView;
    Button1: TButton;
    ImageList1: TImageList;
    SelectedLabel: TLabel;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure DirTreeExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure DirTreeClick(Sender: TObject);
    procedure DirTreeChange(Sender: TObject; Node: TTreeNode);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    VolumeLetter : array ['a'..'z'] of String;
    MountPoints : TStringList;
    MountVolumes : TStringList;

    function BuildPath(Node : TTreeNode) : String;
    procedure ExpandNode(Node : TTreeNode);

  end;

var
  BrowsePathDlg: TBrowsePathDlg;

implementation

uses install, global;

{$R *.DFM}

procedure TBrowsePathDlg.FormCreate(Sender: TObject);
var
   Drive : Char;
   Buffer : String;
   DriveString : String;
   i : Integer;

   Component : DWORD;
   Flags : DWORD;
   NewItem : TTreeNode;
begin
   // Find any drive letter assignments...
   for Drive := 'a' to 'z' do
   begin
      DriveString := Drive + ':\';

      SetLength(Buffer, 1024);
      if JGetVolumeNameForVolumeMountPoint(PChar(DriveString), PChar(Buffer), Length(Buffer)) then
      begin
         SetLength(Buffer, strlen(PChar(Buffer)));
         if Length(Buffer) > 0 then
         begin
            Buffer := Copy(Buffer, 12, Length(Buffer) - 13);
            VolumeLetter[Drive] := Buffer;
            Debug(DriveString + ' = ' + Buffer);
         end;
      end;
   end;

   // Find any mount points
   MountPoints := TStringList.Create;
   MountVolumes := TStringList.Create;
   MainForm.GetListOfMountPoints(MountPoints);
   for i := 0 to MountPoints.Count - 1 do
   begin
      if JGetVolumeNameForVolumeMountPoint(PChar(MountPoints[i]), PChar(Buffer), Length(Buffer)) then
      begin
         SetLength(Buffer, StrLen(PChar(Buffer)));
         if Length(Buffer) > 0 then
         begin
            Buffer := Copy(Buffer, 12, Length(Buffer) - 13);
         end;
         MountVolumes.Add(Buffer);
      end
      else
      begin
         MountVolumes.Add('');
      end;
      Debug('MP = ' + MountPoints[i] + ' = ' + MountVolumes[i]);
   end;

   // add the drives to the tree...
   for Drive := 'a' to 'z' do
   begin
      if Length(VolumeLetter[Drive]) > 0 then
      begin
         DriveString := UpperCase(Drive + ':\');
         SetLength(Buffer, 1024);
         if GetVolumeInformation(PChar(DriveString), nil, 0, nil, Component, Flags, PChar(Buffer), Length(Buffer)) then
         begin
            SetLength(Buffer, Strlen(PChar(Buffer)));
            if (Flags and FILE_SUPPORTS_REPARSE_POINTS) = FILE_SUPPORTS_REPARSE_POINTS then
            begin
               NewItem := DirTree.Items.Add(nil, DriveString);
               NewItem.HasChildren := True;
               NewItem.ImageIndex := 0;
               NewItem.SelectedIndex := 0;
            end;
         end;
      end;
   end;
end;

function TBrowsePathDlg.BuildPath(Node : TTreeNode) : String;
begin
   if Assigned(Node.Parent) then
   begin
      Result := BuildPath(Node.Parent) + Node.Text;
   end
   else
   begin
      Result := Node.Text;
   end;

   if Length(Result) > 0 then
   begin
      if Result[Length(Result)] <> '\' then
      begin
         Result := Result + '\';
      end;
   end;
end;

procedure TBrowsePathDlg.ExpandNode(Node : TTreeNode);
var
   Path        : String;
   FindData    : TWIN32FindData;
   FindHandle  : THandle;
   NewNode     : TTreeNode;
   FileCount   : Integer;
begin
   if Node.HasChildren and (Node.Count = 0) then
   begin
      Global.Wait;
      try
         Path := BuildPath(Node) + '*.*';

         FileCount := 0;
         // do a directory Listing...
         FindHandle := FindFirstFile(PChar(Path), FindData);
         while FindHandle <> INVALID_HANDLE_VALUE do
         begin
            if FindNextFile(FindHandle, FindData) then
            begin
               if (FindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) = FILE_ATTRIBUTE_DIRECTORY then
               begin
                  if (CompareStr(FindData.cFileName, '.') = 0) or (FindData.cFileName = '..') then
                  begin
                     // skip
                  end
                  else
                  begin
                     NewNode := Node.Owner.AddChild(Node, FindData.cFileName);
                     NewNode.HasChildren := True;

                     if ((FindData.dwFileAttributes and FILE_ATTRIBUTE_REPARSE_POINT) = FILE_ATTRIBUTE_REPARSE_POINT) and (FindData.dwReserved0 = IO_REPARSE_TAG_MOUNT_POINT) then
                     begin
                        NewNode.ImageIndex := 1;
                        NewNode.SelectedIndex := 1;
                     end
                     else
                     begin
                        NewNode.ImageIndex := 2;
                        NewNode.SelectedIndex := 2;
                     end;
                  end;
               end
               else
               begin
                  Inc(FileCount);
               end;
            end
            else
            begin
               Windows.FindClose(FindHandle);
               FindHandle := INVALID_HANDLE_VALUE;
            end;
         end;
         if (Node.Count + FileCount) = 0 then
         begin
            if Node.ImageIndex = 2 then
            begin
               Node.ImageIndex := 3;
               Node.SelectedIndex := 3;
            end;
         end;
      finally
         Global.WaitDone;
      end;
   end;
   if Node.Count = 0 then
   begin
      Node.HasChildren := False;
   end;
end;


procedure TBrowsePathDlg.DirTreeExpanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
begin
   ExpandNode(Node);
   if Node.Count = 0 then
   begin
      AllowExpansion := False;
   end;
end;

procedure TBrowsePathDlg.DirTreeClick(Sender: TObject);
begin
   if Assigned(DirTree.Selected) then
   begin
      ExpandNode(DirTree.Selected);

      if DirTree.Selected.ImageIndex = 3 then
      begin
         DirTreeChange(DirTree, DirTree.Selected);
      end;
   end;
end;

procedure TBrowsePathDlg.DirTreeChange(Sender: TObject; Node: TTreeNode);
begin
   if (Node.ImageIndex = 1) or (Node.ImageIndex = 3) then
   begin
      SelectedLabel.Caption := BuildPath(Node);
      OKBtn.Enabled := True;
   end
   else
   begin
      SelectedLabel.Caption := '';
      OKBtn.Enabled := False;
   end;
end;

procedure TBrowsePathDlg.Button1Click(Sender: TObject);
var
   Node     : TTreeNode;
   NewDir   : String;
begin
   Node := DirTree.Selected;
   if Assigned(Node) then
   begin
      if InputQuery('New Folder', 'Create new folder under ' + BuildPath(Node), NewDir) then
      begin
         if CreateDir(BuildPath(Node) + NewDir) then
         begin
            Node.DeleteChildren;
            Node.HasChildren := True;
            ExpandNode(Node);
         end
         else
         begin
            MessageDlg('Could not create directory called ''' + NewDir + '''.  The error was ' + SysErrorMessage(GetLastError), mtError, [mbOK], 0);
         end;
      end;
   end;
end;

end.
